@@ -1,4 +1,4 @@
-// $Id: wrappers.h,v 1.3 2003/05/26 21:49:22 ensc Exp $    --*- c++ -*--
+// $Id: wrappers.h,v 1.4 2003/07/10 00:11:22 ensc Exp $    --*- c++ -*--
 
 // Copyright (C) 2002,2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
 //  
@@ -216,6 +216,16 @@ Efork()
   return res;
 }
 
+/*@unused@*/
+inline static int
+Ebind(int sock, void const *addr, int addrlen)
+{
+  int		res = bind(sock, addr, addrlen);
+  FatalErrnoError(res==-1, 1, "bind()");
+
+  return res;
+}
+
 /*@unused@*/
 inline static /*@null@*//*@only@*/ void *
 Erealloc(/*@only@*//*@out@*//*@null@*/ void *ptr,
